@@ -7,7 +7,7 @@ use crate::{error::Closed, IntoReader};
 // that would be bad because `.map` on Stream and `.map` on Eventual have very
 // different semantics. In general, Stream has very different semantics. It even
 // has a size_hint - a Stream is a progressively available Vec (distinct
-// values), but an Eventual is an eventualy consistent and updating "latest"
+// values), but an Eventual is an eventually consistent and updating "latest"
 // value which infers no sequence and may drop intermediate values.
 pub struct EventualReader<T> {
     change: ChangeReader<T>,
@@ -34,16 +34,12 @@ where
 {
     type Output = Result<T, Closed>;
     // TODO: This is currently checking for a pushed value, but that will require
-    // eg: map() to run in separate tasks. It might be desireble to have this poll
+    // eg: map() to run in separate tasks. It might be desirable to have this poll
     // the future that would produce values. But... that may be very complex. A
     // refactor may be necessary.
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        let mut swap = None;
-        self.eventual
-            .change
-            .change
-            .swap_or_wake(&mut swap, &self.eventual.prev, cx);
-        match swap {
+        let update = self.eventual.change.change.poll(&self.eventual.prev, cx);
+        match update {
             None => Poll::Pending,
             Some(value) => {
                 self.eventual.prev = Some(value.clone());
